extends CanvasLayer
var gname = "menu"

const INVRES = preload("res://scripts/inventoryres.gd")
var quickplay_settings = null

## Title Card
func play_title_anime(animation):
	var ta = get_node("title/title_anime")
	if((ta.current_animation != animation) && (ta.has_animation(animation))):
		ta.play(animation)

func play_random_title():
	randomize()
	var ta = get_node("title/title_anime")
	var inti = randi() % ta.get_animation_list().size()
	play_title_anime(ta.get_animation_list()[inti])

##Match Lobby
func toggle_ml_settings(value,instaload = ""):#Displays the Match lobby, This may need to be merged a bit with the other toggle_lobby functions
	var scene = "res://ui/match_lobby.tscn"
	var namescene = "match_lobby"
	match(value):
		0,1:#For match lobby, do nothing
			pass
		(2):#For Multiplayer, display the multiplayer lobby.
			scene = "res://ui/online_lobby.tscn"
			namescene = "online_lobby"
		(3):#For campaign display campaign map
			scene = "res://ui/campaignmap.tscn"
			namescene = "campaignmap"
		(4):#For instantly loading in a specific level
			if(load(instaload) != null):
				get_tree().change_scene_to(load(instaload))
				get_tree().call_group("hud","create_match",quickplay_settings,false)
	if(has_node("lobby_bg")):
		var l_bg = get_node("lobby_bg")
		var over_bg = get_node("bg_overlay")
		if(l_bg.has_node(namescene)):
			var ml = l_bg.get_node(namescene)
			ml.queue_free()
			l_bg.visible = false
			over_bg.visible = false
		else:
			var ml = load(scene).instance()
			if((value == 0) || (value == 1)):#If it's a strait boot to match_lobby
				ml.lobby_mode = value
			l_bg.visible = true
			over_bg.visible = true
			l_bg.add_child(ml)

func close_lobby_bg():#This is to close out the lobby, This may need to be merged a bit with the other toggle_lobby functions
	if(has_node("lobby_bg")):
		var l_bg = get_node("lobby_bg")
		var over_bg = get_node("bg_overlay")#You may be wondering why this is outside the lobby_bg node
		for o in l_bg.get_children():#This is why
			o.queue_free()#If bg_overlay was in here it would be removed.
		l_bg.visible = false
		over_bg.visible = false

func set_qp(new_set):
	quickplay_settings = new_set

func _on_qplay_begin():#Quick play should be merged with match_lobby
	var new_scene = load(quickplay_settings.level)
	get_tree().change_scene_to(new_scene)
	var is_network = get_tree().has_network_peer()#If there is a network game involved
	get_tree().call_group("hud","create_match",quickplay_settings,is_network)

##General Usage
func reprint(text):#Just sends a visible info notice to the main menu.
	get_node("reporter").set_text(str(text))

func reset_popup():#Resets the pop up generally used for inventory management
	var pop = get_node("general_popup")
	for ch in pop.get_node("stuff").get_children():
		ch.queue_free()

	pop.visible = false

##Save and Load
func filebow(is_saving):
	var filebrowse = get_node("file_browser")
	if(is_saving):
		filebrowse.set_mode(filebrowse.MODE_SAVE_FILE)
	else:
		filebrowse.set_mode(filebrowse.MODE_OPEN_FILE)
	filebrowse.popup()

func filebow_confirm():
	var filebrowse = get_node("file_browser")
	var modetype = filebrowse.get_mode()
	var filepath = filebrowse.get_current_file()
	filebrowse.set_visible(false)
	if(modetype == 4):
		save_game(filepath)
	else:
		load_game(filepath)

func save_game(path):
	reprint("Saving")
	var s = load("res://scripts/save_system.gd")
	s.save_game(path)

func load_game(path):
	var s = load("res://scripts/save_system.gd")
	var content = s.load_game(path)
	reprint("Loading")
	if content == null:
		reprint("Loa-ACK! File does not exist!")

##Graphic Options
func o_set_vsync(value):#Enables the dreaded vsync. 
	ProjectSettings.set("window/vsync/use_vsync",value)

func o_set_language(value):
	match(value):
		-1,0,1:#English
			TranslationServer.add_translation(load("res://scripts/translations/translation_server.en.translation"))
			TranslationServer.set_locale("en")
		2:#Russian
			TranslationServer.add_translation(load("res://scripts/translations/translation_server.ru.translation"))
			TranslationServer.set_locale("ru")
		3:#Spanish
			TranslationServer.add_translation(load("res://scripts/translations/translation_server.es.translation"))
			TranslationServer.set_locale("es")
		4:#Japanese
			TranslationServer.add_translation(load("res://scripts/translations/translation_server.ja.translation"))
			TranslationServer.set_locale("ja")
		5:#Polish
			TranslationServer.add_translation(load("res://scripts/translations/translation_server.pl.translation"))
			TranslationServer.set_locale("pl")
		6:#English New Zealand. This is a joke translation.
			TranslationServer.add_translation(load("res://scripts/translations/translation_server.en_NZ.translation"))
			TranslationServer.set_locale("en_NZ")
	print("main: translating - ", value, TranslationServer.get_locale_name(TranslationServer.get_locale()), " - ", TranslationServer.translate("sound"))

##Sound options
func s_set_sound(value):
	ProjectSettings.set("Game/Options/sound_volume",value)
func s_set_music(value):
	ProjectSettings.set("Game/Options/music_volume",value)
	get_tree().call_group("hud","s_set_music_volume")
func s_set_anouncer_db(value):
	ProjectSettings.set("Game/Options/announcer_volume",value)
	get_tree().call_group("hud","s_sound_announcer")
	var a_stat = get_node("TabContainer/options/sound/anounce_stat")
	if value <= 0:
		a_stat.text = "None"
	elif (value > 0) && (value < 15):
		a_stat.text = "Quiet"
	elif (value > 15) && (value < 40):
		a_stat.text = "Whisper"
	elif (value > 40) && (value < 75):
		a_stat.text = "Normal"
	elif (value > 75) && (value < 95):
		a_stat.text = "Loud"
	elif value >= 95:
		a_stat.text = "Bubsy"
	else:
		a_stat.text = "WHAT ARE YOU DOING!"

##FUN OPTIONS
func set_multiplier(value):
	ProjectSettings.set("Game/Fun/multiplier",value)

##CONTROLS
func set_sensitivity(value):
	ProjectSettings.set("Game/Options/Sensitivity",value)

func _on_control_preset_select(indx,playern):#When selecting a preset
	var up = InputEventKey.new()
	var down = InputEventKey.new()
	var left = InputEventKey.new()
	var right = InputEventKey.new()
	var prime = InputEventKey.new()
	var second = InputEventKey.new()
	var decoy = InputEventKey.new()
	var other = InputEventKey.new()
	var buy = InputEventKey.new()
	var command = InputEventKey.new()
	var one = InputEventKey.new()
	var two = InputEventKey.new()
	var three = InputEventKey.new()
	var four = InputEventKey.new()
	var fly = InputEventKey.new()
	var pickup = InputEventKey.new()
	var aim = null

	if indx == 0:#Mouse and Keyboard
		up.set_scancode(KEY_W)
		down.set_scancode(KEY_S)
		left.set_scancode(KEY_A)
		right.set_scancode(KEY_D)
		prime = InputEventMouseButton.new()
		prime.set_button_index(BUTTON_LEFT)
		second = InputEventMouseButton.new()
		second.set_button_index(BUTTON_RIGHT)
		decoy.set_scancode(KEY_SHIFT)
		other.set_scancode(KEY_Q)
		buy.set_scancode(KEY_CONTROL)
		command.set_scancode(KEY_ALT)
		one.set_scancode(KEY_1)
		two.set_scancode(KEY_2)
		three.set_scancode(KEY_3)
		four.set_scancode(KEY_4)
		fly.set_scancode(KEY_SPACE)
		pickup.set_scancode(KEY_F)

	elif indx == 1:#Numpad
		up.set_scancode(KEY_KP_8)
		down.set_scancode(KEY_KP_2)
		left.set_scancode(KEY_KP_4)
		right.set_scancode(KEY_KP_6)
		prime.set_scancode(KEY_KP_0)
		second.set_scancode(KEY_KP_PERIOD)
		decoy.set_scancode(KEY_KP_5)
		other.set_scancode(KEY_KP_MULTIPLY)
		buy.set_scancode(KEY_KP_SUBTRACT)
		command.set_scancode(KEY_KP_ADD)
		one.set_scancode(KEY_KP_1)
		two.set_scancode(KEY_KP_3)
		three.set_scancode(KEY_KP_7)
		four.set_scancode(KEY_KP_9)
		fly.set_scancode(KEY_KP_ENTER)
		pickup.set_scancode(KEY_KP_DIVIDE)
		aim = [InputEventKey.new(),InputEventKey.new(),InputEventKey.new(),InputEventKey.new()]
		aim[0].set_scancode(KEY_KP_8)
		aim[1].set_scancode(KEY_KP_2)
		aim[2].set_scancode(KEY_KP_4)
		aim[3].set_scancode(KEY_KP_6)

	elif indx == 2:#Playstation to PC
		up.set_scancode(KEY_W)
		down.set_scancode(KEY_S)
		left.set_scancode(KEY_A)
		right.set_scancode(KEY_D)
		prime.set_scancode(KEY_H)
		second.set_scancode(KEY_J)
		decoy.set_scancode(KEY_Q)
		other.set_scancode(KEY_E)
		buy.set_scancode(KEY_Y)
		command.set_scancode(KEY_I)
		one.set_scancode(KEY_1)
		two.set_scancode(KEY_2)
		three.set_scancode(KEY_3)
		four.set_scancode(KEY_4)
		fly.set_scancode(KEY_U)
		pickup.set_scancode(KEY_K)
		aim = [InputEventKey.new(),InputEventKey.new(),InputEventKey.new(),InputEventKey.new()]
		aim[0].set_scancode(KEY_W)
		aim[1].set_scancode(KEY_S)
		aim[2].set_scancode(KEY_A)
		aim[3].set_scancode(KEY_D)

	elif indx == 3:#VIM
		up.set_scancode(KEY_K)
		down.set_scancode(KEY_J)
		left.set_scancode(KEY_H)
		right.set_scancode(KEY_L)
		prime = InputEventMouseButton.new()
		prime.set_button_index(BUTTON_LEFT)
		second = InputEventMouseButton.new()
		second.set_button_index(BUTTON_RIGHT)
		decoy.set_scancode(KEY_R)
		other.set_scancode(KEY_M)
		buy.set_scancode(KEY_F)
		command.set_scancode(KEY_E)
		one.set_scancode(KEY_1)
		two.set_scancode(KEY_2)
		three.set_scancode(KEY_3)
		four.set_scancode(KEY_4)
		fly.set_scancode(KEY_B)
		pickup.set_scancode(KEY_A)
		
	elif indx == 4:#Controller
		up = InputEventJoypadMotion.new()
		up.set_axis(JOY_AXIS_2)
		down = InputEventJoypadMotion.new()
		down.set_axis(JOY_AXIS_3)
		left = InputEventJoypadMotion.new()
		left.set_axis(JOY_AXIS_0)
		right = InputEventJoypadMotion.new()
		right.set_axis(JOY_AXIS_1)

		prime = InputEventJoypadButton.new()
		prime.set_button_index(JOY_BUTTON_4)
		second = InputEventJoypadButton.new()
		second.set_button_index(JOY_BUTTON_5)
		decoy = InputEventJoypadButton.new()
		decoy.set_button_index(JOY_BUTTON_3)
		other = InputEventJoypadButton.new()
		other.set_button_index(JOY_BUTTON_7)
		buy = InputEventJoypadButton.new()
		buy.set_button_index(JOY_BUTTON_0)
		command = InputEventJoypadButton.new()
		command.set_button_index(JOY_BUTTON_2)
		one = InputEventJoypadButton.new()
		one.set_button_index(JOY_BUTTON_12)
		two = InputEventJoypadButton.new()
		two.set_button_index(JOY_BUTTON_13)
		three = InputEventJoypadButton.new()
		three.set_button_index(JOY_BUTTON_14)
		four = InputEventJoypadButton.new()
		four.set_button_index(JOY_BUTTON_15)
		fly = InputEventJoypadButton.new()
		fly.set_button_index(JOY_BUTTON_1)
		pickup = InputEventJoypadButton.new()
		pickup.set_button_index(JOY_BUTTON_6)

		aim = [InputEventJoypadMotion.new(),InputEventJoypadMotion.new(),InputEventJoypadMotion.new(),InputEventJoypadMotion.new()]
		aim[0].set_axis(JOY_AXIS_6)
		aim[1].set_axis(JOY_AXIS_7)
		aim[2].set_axis(JOY_AXIS_4)
		aim[3].set_axis(JOY_AXIS_5)

	set_controls_as(playern,up,down,left,right,prime,second,decoy,other,buy,command,one,two,three,four,fly,pickup,aim)

func set_controls_as(playern,up,down,left,right,prime,second,decoy,other,buy,command,one,two,three,four,fly,pickup,disshop,aim = null):#Changes the input maps
	var spln = str(playern)
	var actionlist = {	spln+"charu":up,
						spln+"chard":down,
						spln+"charl":left,
						spln+"charr":right,
						spln+"firep":prime,
						spln+"fires":second,
						spln+"decoy":decoy,
						spln+"other":other,
						spln+"buy":buy,
						spln+"command":command,
						spln+"one":one,
						spln+"two":two,
						spln+"three":three,
						spln+"four":four,
						spln+"fly":fly,
						spln+"pickup":pickup,
						spln+"disshop":disshop}
	var aimtype = 0
	if(aim != null):#aim needs to be an array
		if typeof(aim) == TYPE_ARRAY:
			if(aim.size() == 4):
				if(has_node("/root/hud")):
					get_node("/root/hud").player_control_type[playern] = 1
				actionlist[spln+"aimu"] = aim[0]
				actionlist[spln+"aimd"] = aim[1]
				actionlist[spln+"aiml"] = aim[2]
				actionlist[spln+"aimr"] = aim[3]
				aimtype = 2
	else:
		if(has_node("/root/hud")):
			get_node("/root/hud").player_control_type[playern] = 0
	for a in actionlist:
		if(InputMap.has_action(a)):
			InputMap.erase_action(a)
			InputMap.add_action(a)
		elif(!InputMap.has_action(a)):
			InputMap.add_action(a)
		InputMap.action_add_event(a,actionlist[a])

	#Setup the main menu's look
	set_pcontrols_box(playern,up,down,left,right,prime,second,decoy,other,buy,command,one,two,three,four,fly,pickup,aimtype)

func set_pcontrols_box(playern,up,down,left,right,prime,second,decoy,other,buy,command,one,two,three,four,fly,pickup,disshop,aim = 0):#Displays the change on the controls menu
#	print("main.gd: player "+str(playern)+" control is setup")
	var conplayer = get_node("TabContainer/controls/pscroll/players")
	if(conplayer.has_node("setup_player"+str(playern+1))):
		var playernode = conplayer.get_node("setup_player"+str(playern+1))
		playernode.get_node("name").set_text("Player"+str(playern+1))
		playernode.get_node("updown/input").set_text(str(up.as_text()))
		playernode.get_node("updown/input2").set_text(str(down.as_text()))
		playernode.get_node("leftright/input").set_text(str(left.as_text()))
		playernode.get_node("leftright/input2").set_text(str(right.as_text()))
		playernode.get_node("primesecond/input").set_text(str(prime.as_text()))
		playernode.get_node("primesecond/input2").set_text(str(second.as_text()))
		playernode.get_node("decoyother/input").set_text(str(decoy.as_text()))
		playernode.get_node("decoyother/input2").set_text(str(other.as_text()))
		playernode.get_node("buycommand/input").set_text(str(buy.as_text()))
		playernode.get_node("buycommand/input2").set_text(str(command.as_text()))
		playernode.get_node("1-4/input").set_text(str(one.as_text()))
		playernode.get_node("1-4/input2").set_text(str(two.as_text()))
		playernode.get_node("1-4/input").set_text(str(three.as_text()))
		playernode.get_node("1-4/input2").set_text(str(four.as_text()))
		playernode.get_node("aimfly/input").select(int(aim))
		playernode.get_node("disshop/input").set_text(disshop.as_text())
		playernode.get_node("aimfly/input2").set_text(str(fly.as_text()))
		playernode.get_node("presets/input").set_text(str(pickup.as_text()))

func get_input_map(playern):
	var spln = str(playern)
	var null_input = InputEventKey.new()
	var ret = {	spln+"charu":null_input,
				spln+"chard":null_input,
				spln+"charl":null_input,
				spln+"charr":null_input,
				spln+"firep":null_input,
				spln+"fires":null_input,
				spln+"decoy":null_input,
				spln+"other":null_input,
				spln+"buy":null_input,
				spln+"command":null_input,
				spln+"one":null_input,
				spln+"two":null_input,
				spln+"three":null_input,
				spln+"four":null_input,
				spln+"fly":null_input,
				spln+"disshop":null_input,
				spln+"pickup":null_input}
	for act in ret:
		if InputMap.has_action(act):
			ret[act] = InputMap.get_action_list(act)[0]
	return ret

func setup_player_controls(): #Set up the controls for the player(s) right now it only sets up 4
	var ret = get_input_map(0)
	set_pcontrols_box(0,ret["0charu"],ret["0chard"],ret["0charl"],ret["0charr"],ret["0firep"],ret["0fires"],ret["0decoy"],ret["0other"],ret["0buy"],ret["0command"],ret["0one"],ret["0two"],ret["0three"],ret["0four"],ret["0fly"],ret["0pickup"],ret["0disshop"])
	ret = get_input_map(1)
	set_pcontrols_box(1,ret["1charu"],ret["1chard"],ret["1charl"],ret["1charr"],ret["1firep"],ret["1fires"],ret["1decoy"],ret["1other"],ret["1buy"],ret["1command"],ret["1one"],ret["1two"],ret["1three"],ret["1four"],ret["1fly"],ret["1pickup"],ret["1disshop"])
	ret = get_input_map(2)
	set_pcontrols_box(2,ret["2charu"],ret["2chard"],ret["2charl"],ret["2charr"],ret["2firep"],ret["2fires"],ret["2decoy"],ret["2other"],ret["2buy"],ret["2command"],ret["2one"],ret["2two"],ret["2three"],ret["2four"],ret["2fly"],ret["2pickup"],ret["2disshop"])
	ret = get_input_map(3)
	set_pcontrols_box(3,ret["3charu"],ret["3chard"],ret["3charl"],ret["3charr"],ret["3firep"],ret["3fires"],ret["3decoy"],ret["3other"],ret["3buy"],ret["3command"],ret["3one"],ret["3two"],ret["3three"],ret["3four"],ret["3fly"],ret["3pickup"],ret["3disshop"])
	get_node("TabContainer/controls/pscroll/players/setup_player1/presets/input2").connect("item_selected",self,"_on_control_preset_select",[0])
	get_node("TabContainer/controls/pscroll/players/setup_player2/presets/input2").connect("item_selected",self,"_on_control_preset_select",[1])
	get_node("TabContainer/controls/pscroll/players/setup_player3/presets/input2").connect("item_selected",self,"_on_control_preset_select",[2])
	get_node("TabContainer/controls/pscroll/players/setup_player4/presets/input2").connect("item_selected",self,"_on_control_preset_select",[3])

##Online/LAN
func toggle_disconnect():#For toggling the display of the disconnect button, for online play.
	var dis_but = get_node("disconnect")
	if(get_tree().has_network_peer()):
		dis_but.visible = true
	else:
		close_lobby_bg()
		dis_but.visible = false

##CONNECTIONS
func _on_quit_pressed():
	get_tree().quit()

##MAIN
func _ready():
	add_to_group(gname)
	quickplay_settings = load("res://scripts/match_setting.gd").new()
	var invres = INVRES.new()
	var ep_uni = [invres.get_units()["mimic"],invres.get_units()["motor"],invres.get_units()["bazooka"],invres.get_units()["turrent"],invres.get_units()["airplane"],invres.get_units()["jeep"],invres.get_units()["antiair"],invres.get_units()["tank"],]
	quickplay_settings.level = "res://levels/DB/DB_test.tscn"
	quickplay_settings.music = "res://assets/audio/music/ogg/2fm.ogg"
	quickplay_settings.players = [	
									quickplay_settings.add_player_setting("res://scripts/player.gd", invres.get_mechas()["Prototype"][0], invres.get_decoys()["decoy7"], ep_uni, [invres.get_parts()["NONE"],invres.get_parts()["NONE"]], [invres.get_weapons()["basic"],invres.get_weapons()["basic"]], 1, ["Player1","0000ff"], 1, 0),
									quickplay_settings.add_player_setting("res://scripts/stupid_AI.gd", invres.get_mechas()["Prototype"][0], invres.get_decoys()["decoy7"], ep_uni, [invres.get_parts()["NONE"],invres.get_parts()["NONE"]], [invres.get_weapons()["basic"],invres.get_weapons()["basic"]], 2, ["Player2","ff0000"], 1, 0)
									]
#OPTIONS
	get_node("TabContainer/controls/mou_sense").connect("changed",self,"set_sensitivity")
	setup_player_controls()
	get_node("TabContainer/options/filemanage/load").connect("pressed",self,"filebow",[false])
	get_node("TabContainer/options/filemanage/save").connect("pressed",self,"filebow",[true])
	var filebrowse = get_node("file_browser")
	filebrowse.connect("confirmed",self,"filebow_confirm")
	get_node("TabContainer/options/funstuff/multiplier").connect("value_changed",self,"set_multiplier")
	get_node("TabContainer/options/graphics/vbox/use_vsync").connect("toggled",self,"o_set_vsync")

#LANGUAGE
	o_set_language(0)
	get_node("TabContainer/options/graphics/vbox/language").connect("item_selected",self,"o_set_language")
#Title Card
	get_node("title/title_timer").connect("timeout",self,"play_random_title")
#Sound
	get_node("TabContainer/options/sound/sound").value = ProjectSettings.get("Game/Options/sound_volume")
	get_node("TabContainer/options/sound/sound").connect("value_changed",self,"s_set_sound")
	get_node("TabContainer/options/sound/music").value = ProjectSettings.get("Game/Options/music_volume")
	get_node("TabContainer/options/sound/music").connect("value_changed",self,"s_set_music")
	get_node("TabContainer/options/sound/announcer").value = ProjectSettings.get("Game/Options/announcer_volume")
	get_node("TabContainer/options/sound/announcer").connect("value_changed",self,"s_set_anouncer_db")
#Quick play
	get_node("quickplay").connect("pressed",self,"_on_qplay_begin")
	get_node("TabContainer/options/matches/qp_set").connect("pressed",self,"toggle_ml_settings",[1])
#INVENTORY
#Current none
# GAME
	get_node("bg_overlay/but_close").connect("pressed",self,"close_lobby_bg")
	get_node("TabContainer/game/local_match").connect("pressed",self,"toggle_ml_settings",[0])
	get_node("TabContainer/game/LAN_match").connect("pressed",self,"toggle_ml_settings",[2])
	get_node("TabContainer/game/campaign_match").connect("pressed",self,"toggle_ml_settings",[3])
	get_node("TabContainer/game/tutorial").connect("pressed",self,"toggle_ml_settings",[4,"res://levels/Tutorial.tscn"])
#Online/Lan
	var dis_but = get_node("disconnect")
	dis_but.connect("pressed",get_tree(),"set_network_peer",[null])
	dis_but.connect("pressed",self,"toggle_disconnect")
	toggle_disconnect()